#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <avr/eeprom.h>
#include <stdlib.h>
#include <portbits.h>
#include "global.h"
#include "uart.h"
#include "rf12.h"
#include "leds.h"

#define F_CPU 10000000UL
#include <util/delay.h>

#define RF_BAUDRATE		19200		// Baudrate des RFM12 (Datenrate in kByte=Baudrate/16)
#define UART_BAUDRATE	19200

#define MAX_BUF			128			// Paket Gre in Bytes (maximal 250)
#define TX_TIMEOUT		200			// Maximale Wartezeit auf Daten in ms (max 255)

#define ANSWER_TIMEOUT	50			// Maximale Wartezeit auf die Besttigung der Daten

#define RETRY			10			// Maximale Anzahl an Sendeversuchen

volatile unsigned char delaycnt;
unsigned char rxbuf[MAX_BUF];		// Puffer fr empfangene Daten
unsigned char txbuf[MAX_BUF];		// Puffer fr zu sendende Daten

void rx_packet(void);
void tx_packet(unsigned char packet_size);

int main(void)
{	PORTD=254;
	DDRD=254;
	DDRC=255;
    uart_init(UART_BAUD_SELECT(UART_BAUDRATE, F_CPU));
	rf12_init();					// ein paar Register setzen (z.B. CLK auf 10MHz)
	rf12_setfreq(RF12FREQ(433.92));	// Sende/Empfangsfrequenz auf 433,92MHz einstellen
	rf12_setbandwidth(4, 0, 1);		// 200kHz Bandbreite, 0dB Verstrkung, DRSSI threshold: -97dBm 
	rf12_setbaud(RF_BAUDRATE);		// Baudrate
	rf12_setpower(0, 6);			// 6mW Ausgangangsleistung, 120kHz Frequenzshift

	TCCR1A=0;
	TCCR1B=(1<<WGM12)|1;
	OCR1A=((F_CPU+500)/1000)-1;
	TIMSK=(1<<OCIE1A);

	sei();
	unsigned char tx_cnt=0;

	rf12_rxmode();
	LED_READY=1;

	for (;;)
	{	if (rf12_data())								// Daten im RF12 RX Puffer ?
		{	rx_packet();
		}
		if (uart_data())								// Daten im UART RX Puffer ?
		{	if (!tx_cnt)
				delaycnt=TX_TIMEOUT;					// Nach dem ersten Byte: timeout starten
			txbuf[tx_cnt++]=uart_getchar();
		}
		if (((tx_cnt)&&(delaycnt==0))||(tx_cnt>=MAX_BUF)) // Puffer voll, oder timeout seit erstem Byte im Puffer vorbei ?  -> senden
		{	tx_packet(tx_cnt);
			tx_cnt=0;
		}
	}
}

void rx_packet(void)
{	static unsigned char rx_lastid=255;
	unsigned char rx_cnt,i, rx_id;

	rx_cnt=rf12_rxdata(rxbuf, &rx_id);			// Ja -> komplettes Paket empfangen
	if (rx_cnt>MAX_BUF)							// Checksum error
	{	LED_ERR=1;
		rf12_stoprx();							// auf TX umschalten
		_delay_ms(1);
		rf12_txdata(0, RECEIVED_FAIL, 0);		// Status senden
		rf12_rxmode();							// wieder auf RX umschalten
	}
	else
	{	LED_ERR=0;
		rf12_stoprx();							// auf TX umschalten
		_delay_ms(1);
		rf12_txdata(0, RECEIVED_OK, 0);			// Status senden
		rf12_rxmode();							// wieder auf RX umschalten
		if (rx_id!=rx_lastid)
		{	for (i=0; i<rx_cnt; i++)
				uart_putc(rxbuf[i]);
			rx_lastid=rx_id;
		}
	}
}

void tx_packet(unsigned char packet_size)
{	static unsigned char tx_id=0;
	unsigned char i, rx_cnt;

	for (i=0; i<RETRY; i++)
	{	rf12_stoprx();							// auf TX umschalten			
		_delay_ms(1);
		rf12_txdata(txbuf, packet_size, tx_id);	// senden
		rf12_rxmode();							// wieder auf RX umschalten
		delaycnt=ANSWER_TIMEOUT;
		while (delaycnt)
		{	if (rf12_data())					// Daten im RF12 RX Puffer ?
			{	rx_cnt=rf12_rxdata(rxbuf, &rxbuf[MAX_BUF-1]); // Antwort empfangen
				if (rx_cnt>MAX_BUF)				// Antwort ist Status ?
				{	rxbuf[0]+=rx_cnt;
					if (rxbuf[0]==0)			// Status Code Checksum OK ?
					{	if (rx_cnt==RECEIVED_OK) // Besttigung fr Daten erfolgreich empfangen
						{	goto skiploop;
						}
						break;
					}
				}
			}
		}
		LED_ERR=1;		// Daten nicht (richtig) empfangen -> Fehler Led an
	}
skiploop:
	if (i<RETRY)
		LED_ERR=0;		// Daten richtig bertragen -> Fehler Led aus
	tx_id++;
}

ISR(SIG_OUTPUT_COMPARE1A)					// 1kHz Interrupt
{	
	if (delaycnt)
		delaycnt--;
}



